@@ -1,5 +1,6 @@
 //! provides diffing algorithm which returns patches
 #![allow(clippy::type_complexity)]
+#![allow(unused)]
 use crate::{
     node::attribute::group_attributes_per_name, Attribute, Element, Node,
     Patch, TreePath,
@@ -44,7 +45,7 @@ mod keyed_elements;
 ///     diff,
 ///     vec![Patch::remove_node(
 ///         Some(&"div"),
-///         TreePath::new(vec![0, 0]),
+///         TreePath::new(vec![ 0]),
 ///     )
 ///     ]
 /// );
@@ -56,7 +57,7 @@ pub fn diff_with_key<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -64,7 +65,7 @@ where
     diff_recursive(
         old_node,
         new_node,
-        &[0],
+        &[],
         key,
         &|_old, _new| false,
         &|_old, _new| false,
@@ -72,8 +73,8 @@ where
 }
 
 /// calculate the difference of 2 nodes
-/// if the skip function evaluates to true diffinf of
-/// the node will be skipped entirely
+/// if the skip function evaluates to true, then diffing of
+/// the node and all of it's descendant will be skipped entirely and then proceed to the next node.
 ///
 /// The SKIP fn is passed to check whether the diffing of the old and new element should be
 /// skipped, and assumed no changes. This is for optimization where the developer is sure that
@@ -91,7 +92,7 @@ pub fn diff_with_functions<'a, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -105,17 +106,36 @@ where
         &'a Node<NS, TAG, LEAF, ATT, VAL>,
     ) -> bool,
 {
-    diff_recursive(old_node, new_node, &[0], key, skip, rep)
+    diff_recursive(old_node, new_node, &[], key, skip, rep)
 }
 
-/// returns true if any of the node children has key in their attributes
+/// returns true if all of the node children has key in their attributes
+#[allow(unused)]
+fn is_all_children_keyed<NS, TAG, LEAF, ATT, VAL>(
+    element: &Element<NS, TAG, LEAF, ATT, VAL>,
+    key: &ATT,
+) -> bool
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
+    LEAF: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+{
+    element
+        .get_children()
+        .iter()
+        .all(|child| is_keyed_node(child, key))
+}
+
+/// returns true if any of the children of this element has key in their attributes
 fn is_any_children_keyed<NS, TAG, LEAF, ATT, VAL>(
     element: &Element<NS, TAG, LEAF, ATT, VAL>,
     key: &ATT,
 ) -> bool
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -133,7 +153,7 @@ fn is_keyed_node<NS, TAG, LEAF, ATT, VAL>(
 ) -> bool
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -153,7 +173,7 @@ fn should_replace<'a, 'b, NS, TAG, LEAF, ATT, VAL, REP>(
 ) -> bool
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -204,7 +224,7 @@ fn diff_recursive<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
 where
     NS: PartialEq + Clone + Debug,
     LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
     SKIP: Fn(
@@ -221,11 +241,9 @@ where
         return vec![];
     }
 
-    let replace = should_replace(old_node, new_node, key, rep);
-
     let mut patches = vec![];
 
-    if replace {
+    if should_replace(old_node, new_node, key, rep) {
         patches.push(Patch::replace_node(
             old_node.tag(),
             TreePath::new(path.to_vec()),
@@ -250,12 +268,11 @@ where
         }
         // We're comparing two element nodes
         (Node::Element(old_element), Node::Element(new_element)) => {
-            if is_any_children_keyed(old_element, key)
-                || is_any_children_keyed(new_element, key)
-            {
-                // use diff_keyed_elements if the any of the old_element or new_element
-                // wer are comparing contains a key as an attribute
-                let keyed_patches = diff_keyed_elements(
+            let test = is_any_children_keyed(old_element, key)
+                || is_any_children_keyed(new_element, key);
+
+            if test {
+                let keyed_patches = keyed_elements::diff_keyed_elements(
                     old_element,
                     new_element,
                     key,
@@ -307,7 +324,7 @@ fn diff_non_keyed_elements<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
 where
     NS: PartialEq + Clone + Debug,
     LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
     SKIP: Fn(
@@ -319,7 +336,6 @@ where
         &'a Node<NS, TAG, LEAF, ATT, VAL>,
     ) -> bool,
 {
-    let this_cur_path = path.to_vec();
     let mut patches = vec![];
     let attributes_patches =
         create_attribute_patches(old_element, new_element, path);
@@ -330,8 +346,9 @@ where
 
     let min_count = cmp::min(old_child_count, new_child_count);
     for index in 0..min_count {
-        let mut cur_child_path = path.to_vec();
-        cur_child_path.push(index);
+        // if we iterate trough the old elements, a new child_path is created for that iteration
+        let mut child_path = path.to_vec();
+        child_path.push(index);
 
         let old_child = &old_element
             .children
@@ -340,25 +357,16 @@ where
         let new_child =
             &new_element.children.get(index).expect("No new chold node");
 
-        let more_patches = diff_recursive(
-            old_child,
-            new_child,
-            &cur_child_path,
-            key,
-            skip,
-            rep,
-        );
+        let more_patches =
+            diff_recursive(old_child, new_child, &child_path, key, skip, rep);
         patches.extend(more_patches);
     }
 
     // If there are more new child than old_node child, we make a patch to append the excess element
     // starting from old_child_count to the last item of the new_elements
     if new_child_count > old_child_count {
-        let append_children_patch = create_append_children_patch(
-            old_element,
-            new_element,
-            this_cur_path,
-        );
+        let append_children_patch =
+            create_append_children_patch(old_element, new_element, path);
         patches.push(append_children_patch);
     }
 
@@ -374,12 +382,12 @@ where
 fn create_append_children_patch<'a, NS, TAG, LEAF, ATT, VAL>(
     old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    this_cur_path: Vec<usize>,
+    path: &[usize],
 ) -> Patch<'a, NS, TAG, LEAF, ATT, VAL>
 where
     NS: PartialEq + Clone + Debug,
     LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
 {
@@ -392,7 +400,7 @@ where
 
     Patch::append_children(
         &old_element.tag,
-        TreePath::new(this_cur_path.to_vec()),
+        TreePath::new(path.to_vec()),
         append_patch,
     )
 }
@@ -405,7 +413,7 @@ fn create_remove_node_patch<'a, NS, TAG, LEAF, ATT, VAL>(
 where
     NS: PartialEq + Clone + Debug,
     LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
 {
@@ -438,7 +446,7 @@ fn create_attribute_patches<'a, NS, TAG, LEAF, ATT, VAL>(
 where
     NS: PartialEq + Clone + Debug,
     LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
 {
