@@ -16,7 +16,7 @@ fn find_node_with_key<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Option<(usize, &'a Node<NS, TAG, LEAF, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -52,7 +52,7 @@ fn find_matched_new_child<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Option<(usize, &'a Node<NS, TAG, LEAF, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -75,7 +75,7 @@ fn find_child_node_with_idx<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Option<(usize, &'a Node<NS, TAG, LEAF, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -101,7 +101,7 @@ fn get_matched_old_new_idx<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> (Vec<usize>, Vec<usize>)
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -119,7 +119,7 @@ fn get_unmatched_children_node_idx<NS, TAG, LEAF, ATT, VAL>(
 ) -> Vec<(usize, &Node<NS, TAG, LEAF, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -137,7 +137,7 @@ fn build_keyed_elements<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> BTreeMap<usize, (Vec<&'a VAL>, &'a Node<NS, TAG, LEAF, ATT, VAL>)>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -172,7 +172,7 @@ fn build_matched_old_new_keyed<'a, NS, TAG, LEAF, ATT, VAL>(
 >
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -206,20 +206,45 @@ where
     matched_old_new_keyed
 }
 
-/// Reconciliation of keyed elements
+/// diff the old element with new element
 ///
-/// algorithm:
+/// example case:
 ///
-/// Phase1
-///   - prioritize matching elements that has the same key.
-///   - match non-keyed elements according to their node_idx alignment
+///  old            new
 ///
-/// Phase2
-///   - old child elements that are not matched will be removed
-///   - new child elements that are not matched will be inserted or appended
-///     - inserted if the child node_idx <= old elements children
-///     - appended if the child node_idx is > old elements children
+///  (-) 0              10   (+) (will be inserted at before key3)
+///  (-) 1              key5 (+) (will be inserted at before key3, this is not matched at key5, since key3 is matched first and since this new key5 has passed new key3 position)
+///  (-) 2       .----- key3 (*) (will not matched old key3 since we've gone past last matched (key5 at 5) }
+///  (*) key3 <-'       12   (+) (will be inserted at after key3)
+///  (*) key4 <-.       13   (+) (will be inserted at after key3)
+///  (-) key5    `----- key4 (*) (will not matched old key4 since we've gone past last matched (key5 at )
+///  (-) 6              14   (+) (will be inserted at after key4)
+///    * key6 <-------- key6 *
+///  (-) 8              16   (+) will be inserted at after key6)
+///  (-) 9
 ///
+/// Legend:
+/// (-) means will be removed
+/// (+) means will be inserted
+/// (*) means will be patched
+///
+/// Algorithm flow:
+/// - make a BTreeMap for old index and their old key (old_index_key)
+/// - make a BTreeMap for new index and their new key (new_index_key)
+///
+/// - Use a old_index pointer to 0, this will be used to point to the index of the last matched old key (last_matched_old_index)
+/// - Use a new_index pointer to 0, this will be used to point to the index of the last matched new key (last_matched_new_index)
+///
+/// - from the new_index_key, iterate through the new elements to find which old_index which matched the new key starting from  `last_matched_old_index` until it finds a matched.
+///     - old_index_key[last_matched_old_index..]
+///     - if a matched is found (old_key == new_key) take node of the `old_index` and `new_index`
+///         - create a patch which will delete all the old elements from `old_index` to `last_matched_old_index`, using their own patch_path: [path + old_index].
+///         - create a patch which will insert all the new elements from `new_index` to `last_matched_new_index`, using InsertBefore patch_path: [path + last_matched_old_index].
+///         - set `last_matched_old_index` to `old_index`
+///         - set `last_matched_new_index` to `new_index`
+///     - if we have reached the end of the iteration
+///         - create a patch which will delete all the old_elements from `last_matched_old_index` to the last old elements.
+///         - create a patch which will insert all the new_elements from `last_matched_new_index`, using InsertAfter patch_path: [path + last_matched_new_index].
 pub fn diff_keyed_elements<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
     old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
@@ -230,7 +255,7 @@ pub fn diff_keyed_elements<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -398,7 +423,7 @@ fn create_insert_node_patches<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -433,7 +458,7 @@ fn create_append_children_patches<'a, NS, TAG, LEAF, ATT, VAL>(
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
