@@ -62,24 +62,34 @@ mod tree_path;
 pub enum Patch<'a, NS, TAG, LEAF, ATT, VAL>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
 {
-    /// Insert a vector of child nodes to the current node being patch.
-    /// The usize is the index of of the children of the node to be
-    /// patch to insert to. The new children will be inserted before this usize
-    InsertNode {
-        /// the tag of the target node we insert this node into
+    /// insert the nodes before the node at patch_path
+    InsertBeforeNode {
+        /// the tag of the node at patch_path
         tag: Option<&'a TAG>,
-        /// the path to traverse to get to the target element of which our node will be inserted before it.
+        /// the path to traverse to get to the target element
+        /// of which our nodes will be inserted before it.
         patch_path: TreePath,
-        /// the node to be inserted
-        node: &'a Node<NS, TAG, LEAF, ATT, VAL>,
+        /// the nodes to be inserted before patch_path
+        nodes: Vec<&'a Node<NS, TAG, LEAF, ATT, VAL>>,
     },
 
-    /// Append a vector of child nodes to a parent node id.
+    /// insert the nodes after the node at patch_path
+    InsertAfterNode {
+        /// the tag of the node at patch_path
+        tag: Option<&'a TAG>,
+        /// the path to traverse to get to the target element
+        /// of which our nodes will be inserted after it.
+        patch_path: TreePath,
+        /// the nodes to be inserted after the patch_path
+        nodes: Vec<&'a Node<NS, TAG, LEAF, ATT, VAL>>,
+    },
+
+    /// Append a vector of child nodes to a parent node id at patch_path
     AppendChildren {
         /// the tag of the node we are appending the children into
         tag: &'a TAG,
@@ -146,7 +156,7 @@ where
 impl<'a, NS, TAG, LEAF, ATT, VAL> Patch<'a, NS, TAG, LEAF, ATT, VAL>
 where
     NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Clone + Debug,
+    TAG: PartialEq + Debug,
     LEAF: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
@@ -154,7 +164,8 @@ where
     /// return the path to traverse for this patch to get to the target Node
     pub fn path(&self) -> &[usize] {
         match self {
-            Patch::InsertNode { patch_path, .. } => &patch_path.path,
+            Patch::InsertBeforeNode { patch_path, .. } => &patch_path.path,
+            Patch::InsertAfterNode { patch_path, .. } => &patch_path.path,
             Patch::AppendChildren { patch_path, .. } => &patch_path.path,
             Patch::RemoveNode { patch_path, .. } => &patch_path.path,
             Patch::ReplaceNode { patch_path, .. } => &patch_path.path,
@@ -167,7 +178,8 @@ where
     /// return the tag of this patch
     pub fn tag(&self) -> Option<&TAG> {
         match self {
-            Patch::InsertNode { tag, .. } => *tag,
+            Patch::InsertBeforeNode { tag, .. } => *tag,
+            Patch::InsertAfterNode { tag, .. } => *tag,
             Patch::AppendChildren { tag, .. } => Some(tag),
             Patch::RemoveNode { tag, .. } => *tag,
             Patch::ReplaceNode { tag, .. } => *tag,
@@ -183,10 +195,36 @@ where
         patch_path: TreePath,
         node: &'a Node<NS, TAG, LEAF, ATT, VAL>,
     ) -> Patch<'a, NS, TAG, LEAF, ATT, VAL> {
-        Patch::InsertNode {
+        Patch::InsertBeforeNode {
             tag,
             patch_path,
-            node,
+            nodes: vec![node],
+        }
+    }
+
+    /// create an InsertBeforeNode patch
+    pub fn insert_before_node(
+        tag: Option<&'a TAG>,
+        patch_path: TreePath,
+        nodes: Vec<&'a Node<NS, TAG, LEAF, ATT, VAL>>,
+    ) -> Patch<'a, NS, TAG, LEAF, ATT, VAL> {
+        Patch::InsertBeforeNode {
+            tag,
+            patch_path,
+            nodes,
+        }
+    }
+
+    /// create an InsertAfterNode patch
+    pub fn insert_after_node(
+        tag: Option<&'a TAG>,
+        patch_path: TreePath,
+        nodes: Vec<&'a Node<NS, TAG, LEAF, ATT, VAL>>,
+    ) -> Patch<'a, NS, TAG, LEAF, ATT, VAL> {
+        Patch::InsertAfterNode {
+            tag,
+            patch_path,
+            nodes,
         }
     }
 
